@@ -1,9 +1,9 @@
 #![feature(string_remove_matches)]
 
-use std::{path::PathBuf, process::exit};
+use std::{num::NonZeroUsize, path::PathBuf};
 
 use anyhow::Context;
-use clap::{AppSettings, Args, Parser, Subcommand, ValueHint};
+use clap::{Args, Parser, Subcommand, ValueHint};
 use clap_num::si_number;
 use clap_verbosity_flag::Verbosity;
 use cli_errors::{CliExitAnyhowWrapper, CliExitError, CliResult};
@@ -13,9 +13,8 @@ use ftzz::generator::{Generator, GeneratorBuilder};
 /// A random file and directory generator
 #[derive(Parser, Debug)]
 #[clap(version, author = "Alex Saveau (@SUPERCILEX)")]
-#[clap(global_setting(AppSettings::InferSubcommands))]
-#[clap(global_setting(AppSettings::UseLongFormatForHelpSubcommand))]
-#[cfg_attr(test, clap(global_setting(AppSettings::HelpExpected)))]
+#[clap(infer_subcommands = true)]
+#[cfg_attr(test, clap(help_expected = true))]
 struct Ftzz {
     #[clap(flatten)]
     verbose: Verbosity,
@@ -54,7 +53,7 @@ struct Generate {
     /// generated so long as we attempt to get close to N.
     #[clap(short = 'n', long = "files", alias = "num-files")]
     #[clap(parse(try_from_str = num_files_parser))]
-    num_files: usize,
+    num_files: NonZeroUsize,
 
     /// Whether or not to generate exactly N files
     #[clap(long = "files-exact")]
@@ -89,7 +88,7 @@ struct Generate {
     /// files).
     #[clap(short = 'r', long = "ftd-ratio")]
     #[clap(parse(try_from_str = file_to_dir_ratio_parser))]
-    file_to_dir_ratio: Option<usize>,
+    file_to_dir_ratio: Option<NonZeroUsize>,
 
     /// Change the PRNG's starting seed
     ///
@@ -132,10 +131,10 @@ mod generate_tests {
     fn params_are_mapped_correctly() {
         let options = Generate {
             root_dir: PathBuf::from("abc"),
-            num_files: 373,
+            num_files: NonZeroUsize::new(373).unwrap(),
             num_bytes: 637,
             max_depth: 43,
-            file_to_dir_ratio: Some(37),
+            file_to_dir_ratio: Some(NonZeroUsize::new(37).unwrap()),
             seed: 775,
             files_exact: false,
             bytes_exact: false,
@@ -163,7 +162,7 @@ mod generate_tests {
             exact: global_exact,
 
             root_dir: PathBuf::new(),
-            num_files: 1,
+            num_files: NonZeroUsize::new(1).unwrap(),
             num_bytes: 0,
             max_depth: 0,
             file_to_dir_ratio: None,
@@ -187,7 +186,7 @@ mod generate_tests {
             exact: global_exact,
 
             root_dir: PathBuf::new(),
-            num_files: 1,
+            num_files: NonZeroUsize::new(1).unwrap(),
             num_bytes: 0,
             max_depth: 0,
             file_to_dir_ratio: None,
@@ -202,28 +201,32 @@ mod generate_tests {
     }
 }
 
-fn main() {
-    if let Err(e) = wrapped_main() {
-        if let Some(source) = e.source {
-            eprintln!("{:?}", source);
-        }
-        exit(e.code);
-    }
-}
-
-fn wrapped_main() -> CliResult<()> {
+#[cli_errors::main]
+fn main() -> CliResult<()> {
     let args = Ftzz::parse();
+
+    #[cfg(not(feature = "trace"))]
     simple_logger::init_with_level(args.verbose.log_level().unwrap()).unwrap();
+    #[cfg(feature = "trace")]
+    let _guard = {
+        use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};
+
+        let (chrome_layer, guard) = tracing_chrome::ChromeLayerBuilder::new()
+            .include_args(true)
+            .build();
+        tracing_subscriber::registry().with(chrome_layer).init();
+        guard
+    };
 
     match args.cmd {
         Cmd::Generate(options) => Generator::try_from(options)?.generate(),
     }
 }
 
-fn num_files_parser(s: &str) -> Result<usize, String> {
+fn num_files_parser(s: &str) -> Result<NonZeroUsize, String> {
     let files = lenient_si_number(s)?;
     if files > 0 {
-        Ok(files)
+        Ok(unsafe { NonZeroUsize::new_unchecked(files) })
     } else {
         Err(String::from("At least one file must be generated."))
     }
@@ -233,10 +236,10 @@ fn num_bytes_parser(s: &str) -> Result<usize, String> {
     lenient_si_number(s)
 }
 
-fn file_to_dir_ratio_parser(s: &str) -> Result<usize, String> {
+fn file_to_dir_ratio_parser(s: &str) -> Result<NonZeroUsize, String> {
     let ratio = lenient_si_number(s)?;
     if ratio > 0 {
-        Ok(ratio)
+        Ok(unsafe { NonZeroUsize::new_unchecked(ratio) })
     } else {
         Err(String::from("Cannot have no files per directory."))
     }
@@ -257,7 +260,7 @@ mod cli_tests {
 
     #[test]
     fn verify_app() {
-        Ftzz::into_app().debug_assert();
+        Ftzz::command().debug_assert();
     }
 }
 
@@ -278,13 +281,13 @@ mod cli_generate_tests {
             let f = Ftzz::try_parse_from($args);
 
             assert!(f.is_err());
-            assert_eq!(f.unwrap_err().kind, $error);
+            assert_eq!(f.unwrap_err().kind(), $error);
         };
     }
 
     macro_rules! expect_success {
         ($args:expr) => {{
-            let m = Ftzz::into_app().get_matches_from($args);
+            let m = Ftzz::command().get_matches_from($args);
             <Generate as FromArgMatches>::from_arg_matches(
                 m.subcommand_matches("generate").unwrap(),
             )
@@ -310,7 +313,7 @@ mod cli_generate_tests {
         let g = expect_success!(vec!["ftzz", "generate", "-n", "1", "dir"]);
 
         assert_eq!(g.root_dir, PathBuf::from("dir"));
-        assert_eq!(g.num_files, 1);
+        assert_eq!(g.num_files.get(), 1);
         assert_eq!(g.max_depth, 5);
         assert_eq!(g.file_to_dir_ratio, None);
         assert_eq!(g.seed, 0);
@@ -329,35 +332,35 @@ mod cli_generate_tests {
     fn generate_num_files_accepts_plain_nums() {
         let g = expect_success!(vec!["ftzz", "generate", "--files", "1000", "dir"]);
 
-        assert_eq!(g.num_files, 1000);
+        assert_eq!(g.num_files.get(), 1000);
     }
 
     #[test]
     fn generate_short_num_files_accepts_plain_nums() {
         let g = expect_success!(vec!["ftzz", "generate", "-n", "1000", "dir"]);
 
-        assert_eq!(g.num_files, 1000);
+        assert_eq!(g.num_files.get(), 1000);
     }
 
     #[test]
     fn generate_num_files_accepts_si_numbers() {
         let g = expect_success!(vec!["ftzz", "generate", "--files", "1K", "dir"]);
 
-        assert_eq!(g.num_files, 1000);
+        assert_eq!(g.num_files.get(), 1000);
     }
 
     #[test]
     fn generate_num_files_accepts_commas() {
         let g = expect_success!(vec!["ftzz", "generate", "--files", "1,000", "dir"]);
 
-        assert_eq!(g.num_files, 1000);
+        assert_eq!(g.num_files.get(), 1000);
     }
 
     #[test]
     fn generate_num_files_accepts_underscores() {
         let g = expect_success!(vec!["ftzz", "generate", "--files", "1_000", "dir"]);
 
-        assert_eq!(g.num_files, 1000);
+        assert_eq!(g.num_files.get(), 1000);
     }
 
     #[test]
@@ -402,14 +405,14 @@ mod cli_generate_tests {
             "dir",
         ]);
 
-        assert_eq!(g.file_to_dir_ratio, Some(1000));
+        assert_eq!(g.file_to_dir_ratio, Some(NonZeroUsize::new(1000).unwrap()));
     }
 
     #[test]
     fn generate_short_ratio_accepts_plain_nums() {
         let g = expect_success!(vec!["ftzz", "generate", "-r", "321", "-n", "1", "dir"]);
 
-        assert_eq!(g.file_to_dir_ratio, Some(321));
+        assert_eq!(g.file_to_dir_ratio, Some(NonZeroUsize::new(321).unwrap()));
     }
 
     #[test]
@@ -424,7 +427,7 @@ mod cli_generate_tests {
             "dir",
         ]);
 
-        assert_eq!(g.file_to_dir_ratio, Some(1000));
+        assert_eq!(g.file_to_dir_ratio, Some(NonZeroUsize::new(1000).unwrap()));
     }
 
     #[test]
@@ -439,7 +442,7 @@ mod cli_generate_tests {
             "dir",
         ]);
 
-        assert_eq!(g.file_to_dir_ratio, Some(1000));
+        assert_eq!(g.file_to_dir_ratio, Some(NonZeroUsize::new(1000).unwrap()));
     }
 
     #[test]
@@ -454,7 +457,7 @@ mod cli_generate_tests {
             "dir",
         ]);
 
-        assert_eq!(g.file_to_dir_ratio, Some(1000));
+        assert_eq!(g.file_to_dir_ratio, Some(NonZeroUsize::new(1000).unwrap()));
     }
 
     #[test]
